; *********************************************************************
; *                                                                   *
; *     Komunikacna kniznica wBus, pre AVR miktrokontrolery           *
; *     Napisal: Vladimir Visnovsky                                   *
; *                                                                   *
; *********************************************************************

; Procedury / Funkcie
; bs_Init
; bs_Reset
; bs_Instruction
; bs_Master
; bs_Decode ( vystup -> dbgFlag : Byte; Alebo aj v Acc )
; bs_SendByte [Byte]
; bs_ReceiveByte ( vystup -> Acc : Byte )
; bs_Control ( pri chybe ukoncuje proceduru RET )
; bs_Server (pri detekovani instrukcii vola "dbgEvent_Instruction")
; bs_SendFromFlash ( Z: Adresa Flash )


; Nastavenie Portu a Pinu
.Equ bsPin = 3
.Equ bsPort= PortC
.Def bsFlag= R14
; 0: H/L
; 1: Platne data H/L
; 2: Instrukcia
; 3: Master
; 4: Reset
; 5: Chyba


; Nastavenie rozhrania
.Equ bsSend_Instruction	= 5
.Equ bsSend_L				= 10
.Equ bsSend_H				= 15
.Equ bsSend_Master			= 20
.Equ bsSend_Reset			= 250
.Equ bsReceive_Instruction	= 2
.Equ bsReceive_L			= 7
.Equ bsReceive_H			= 12
.Equ bsReceive_Master		= 17
.Equ bsReceive_Reset		= 22

 ; Skok na koniec
 Rjmp bs_End
 
; Zakladne makra pre komunikaciu
 ; Makro nastavenie portu
 .Macro bs_SetPort
  Cbi bsPort-1	, bsPin
  Cbi bsPort    , bsPin
 .EndMacro
 ; Urovne
 .Macro bs_L
  Sbi bsPort-1 , bsPin
 .EndMacro  
 .Macro bs_H
  Cbi bsPort-1 , bsPin
 .EndMacro 
 ; Cakanie 
 .Macro bs_Wait
  Nop ; 1Mhz
  Nop
  Nop
  Nop ; 4Mhz
  Nop
  Nop
  Nop
  Nop ; 8Mhz
  Nop ; 1Mhz
  Nop
  Nop
  Nop ; 4Mhz
  Nop
  Nop
  Nop
  Nop ; 8Mhz
  Nop
  Nop
 .EndMacro 
 
; Inicializacna procedura
 abs_Init:
  bs_SetPort
  Acc [0]
  Mov bsFlag	, A
  Ret
 .Macro bs_Init
  Rcall abs_Init
 .EndMacro
 
; Odoslanie bitu (parameter je A)
 abs_SendBite:
  Cli
  Mov T			, A     
  ; Zistenie stavu zbernice a pockanie, kym sa uvolni
  Sbis bsPort-2, bsPin
   Rjmp abs_SendBite
  ; Zbernica je volna. Stiahnutie zbernice na L 
  bs_L
  ; Cakacia slucka, na kolko ma byt zbernica stiahnuta
  abs_SendBite_Loop:
   bs_Wait
   Nop
   Dec T
   Brne abs_SendBite_Loop
  bs_H 
  ; Cakanie na odozvu
  abs_SendBite_WaitLoop:
   bs_Wait
   Dec T
   Breq abs_SendBite
   ; Zistenie stavu zbernice. Pokracuj ked sa stiahne
   Sbic bsPort-2, bsPin
    Rjmp abs_SendBite_WaitLoop
  ; Koniec
  ;Cbi PortC	, 5
  Sei
  Ret  
 .Macro bs_SendBite
  Rcall abs_SendBite
 .EndMacro
     
; Prijimanie bitu (parameter je A)
 abs_ReciveBite:
  Cli
  ; Zistenie stavu zbernice a pockanie, kym sa uvolni
  Sbis bsPort-2, bsPin
   Rjmp abs_ReciveBite
  ; Zbernica je volna. Cakaj na zaciatok
  Clr A
  abs_ReciveBite_WaitLoop:
  Sbic bsPort-2, bsPin
   Rjmp abs_ReciveBite_WaitLoop
  ; Cakacia slucka, na kolko je zbernica stiahnuta
  abs_ReciveBite_Loop:
   bs_Wait
   Inc A
   Sbis bsPort-2, bsPin
   Rjmp abs_ReciveBite_Loop
  ; Odozva
  bs_Wait
  bs_Wait
  bs_L
  bs_Wait
  bs_Wait
  bs_Wait
  bs_H
  ; Koniec
  Sei
  Ret  
 .Macro bs_ReciveBite
  Rcall abs_ReciveBite
 .EndMacro
   
; Komunikacia
 ; Reset
 .Macro bs_Reset
  Acc [bsSend_Reset]
  Rcall abs_SendBite
 .EndMacro 
 ; Instrukcia
 .Macro bs_Instruction
  Acc [bsSend_Instruction]
  Rcall abs_SendBite
 .EndMacro 
 ; Master
 .Macro bs_Master
  Acc [bsSend_Master]
  Rcall abs_SendBite
 .EndMacro 

; Dekodovanie 
 abs_Decode:
  ; nacitanie bitu
  Rcall abs_ReciveBite
  ; Rozdelenie
  Cpi A		, bsReceive_Reset
   Brsh abs_Decode_Reset
  Cpi A		, bsReceive_Master
   Brsh abs_Decode_Master
  Cpi A		, bsReceive_H
   Brsh abs_Decode_H
  Cpi A		, bsReceive_L
   Brsh abs_Decode_L
  Cpi A		, bsReceive_Instruction
   Brsh abs_Decode_Instruction
  ; Ak je imulz prilis kratky. Priznak chyby
  Acc [0b00100000]  
  Mov bsFlag , A
  Ret
 ; Reset
 abs_Decode_Reset:
  Acc [0b00010000]  
  Mov bsFlag , A
  ; Reset
  ; Rcall dbgEvent_Reset
  Ret
 ; Master 
 abs_Decode_Master:  
  Mov  A , bsFlag
  Andi A , 0b00001100
  Ori  A , 0b00001000
  Mov bsFlag , A
  Ret
 ; Instruction
 abs_Decode_Instruction:  
  Mov  A , bsFlag
  Andi A , 0b00001100
  Ori  A , 0b00000100
  Mov bsFlag , A
  Ret
 ; Uroven L/H
 abs_Decode_L:  
  Mov  A , bsFlag
  Andi A , 0b00001110
  Ori  A , 0b00000010
  Mov bsFlag , A
  Ret
 abs_Decode_H:  
  Mov  A , bsFlag
  Andi A , 0b00001111
  Ori  A , 0b00000011
  Mov bsFlag , A
  Ret
 .Macro bs_Decode
  Rcall abs_Decode
 .EndMacro  
  
; Odoslanie Bytu
 abs_SendByte:
  ; Ulozenie bytu
  Push A
  Push B
  Push T
  Mov  B	, A
  Ldi  T	, 8 
  ; Cyklus
  abs_SendByte_Loop:
   Acc [bsSend_H]
   Sbrs B	, 7	
    Acc [bsSend_L]
   ; Odoslanie bitu
   Push T
   bs_SendBite
   Pop T
   ; Posuv bitov
   Lsl B
   ; Pocet odoslanich bitov
   Dec T
   Brne abs_SendByte_Loop
  ; Vytiahnutie
  Pop T
  Pop B 
  Pop A  
  Ret
  .Macro bs_SendByte
   Rcall abs_SendByte
  .EndMacro
  .Macro bs_SendByte_i
   Acc [@0]
   Rcall abs_SendByte
  .EndMacro
    
; Prijem bytu
 abs_ReceiveByte:
  Push B
  Push T
  Ldi  T 		, 8
  ; Slucka nacitavania
  abs_ReceiveByte_Loop:
   ; Nacitanie bytu
   Push B
   Push T
   bs_Decode
   Pop T
   Pop B
   ; Zistenie stavu reset
   Sbrc A		, 4
    Rjmp abs_ReceiveByte_End
   ; Ak bit nieje H/L 
   Sbrs A		, 1
    Rjmp abs_ReceiveByte_Error
   ; Precitany byte:
   Lsl B
   Bst A		, 0
   Bld B		, 0
   ; Dlzka citania bitov
   Dec T
   Brne  abs_ReceiveByte_Loop
  ; Koniec slucky
  Mov A		, B
  abs_ReceiveByte_End:
  Pop T
  Pop B
  Ret   
  ; Chyba pri citani
  abs_ReceiveByte_Error:
   Acc [0b00100000]  
   Mov bsFlag , A
  Rjmp abs_ReceiveByte_End
  .Macro bs_ReceiveByte
   RCall abs_ReceiveByte
  .EndMacro
  
; Macro, pre odchod z procedur
 .Macro bs_Control
   ; Ukonci ked je reset
   Sbrc bsFlag	, 4
    Ret
   ; Ukonci ked je chyba
   Sbrc bsFlag	, 5
    Ret
 .EndMacro
   
; Server - obsluha prijatych dat
 abs_Server:
  ; Nacitanie dat
  bs_Decode
  ; Rozdelenie
  Sbrc bsFlag	, 2
   Rjmp abs_Server_Instruction  
  Sbrc bsFlag	, 4
   Rjmp abs_Server_Reset  
 Ret
 abs_Server_Reset:
   ; Rjmp dbgEvent_Reset
  Ret
 ; Instrukcia
 abs_Server_Instruction:
  ; Nacitanie bytu instrukcie
  bs_ReceiveByte
  bs_Control
  ; Rozdelenie 
  ;Rjmp bsEvent_Instruction
  Ret 
 .Macro bs_Server
  RCall abs_Server
 .EndMacro   
   
; Procedura vypisu retzaca z programovej pamate Flash
absWriteFromFlash:
 ; Nacitanie z pamate Flash
 Lpm	A		, Z+
 ;.db 0b10010001, 0b00000101
 ; Ak je znak 00h je koniec retazca
 Tst	A
 Breq	absWriteFromFlashEnd
 ; ak nieje koniec vykresli znak
 bs_SendByte
 RJmp	absWriteFromFlash
 ; KoniecProcedury
 absWriteFromFlashEnd: 
 Ret
; V podobe makra
.MACRO bs_SendFromFlash
 RCall absWriteFromFlash				; Volanie procedury
.ENDMACRO
  
 bs_End:
